package org.nakedobjects.nof.reflect.java;

import org.apache.log4j.Logger;
import org.nakedobjects.noa.adapter.NakedObjectLoader;
import org.nakedobjects.nof.core.adapter.map.IdentityAdapterHashMap;
import org.nakedobjects.nof.core.adapter.map.ObjectLoaderImpl;
import org.nakedobjects.nof.core.adapter.map.PojoAdapterHashMap;
import org.nakedobjects.nof.core.system.ObjectLoaderInstaller;
import org.nakedobjects.nof.core.util.NakedObjectConfiguration;


public abstract class AbstractJavaObjectLoaderInstaller implements ObjectLoaderInstaller {
    private final static Logger LOG = Logger.getLogger(AbstractJavaObjectLoaderInstaller.class); 
        
    /**
     * Creates an {@link ObjectLoaderImpl} wired up with a {@link JavaDomainObjectInjector}
     * (as returned by {@link #createDomainObjectInjector()}) and a 
     * {@link JavaDomainObjectContainer} (as returned by {@link #createDomainObjectContainer()}).
     */
    public NakedObjectLoader createObjectLoader(final NakedObjectConfiguration configuration) {
        LOG.info("installing " + this.getClass().getName());
        JavaObjectLoader objectLoader = createObjectLoader();
        objectLoader.setDomainContainer(createDomainObjectContainer());
        objectLoader.setPojoAdapterMap(new PojoAdapterHashMap());
        objectLoader.setIdentityAdapterMap(new IdentityAdapterHashMap());
        return objectLoader;
    }

    /**
     * Subclasses can override if required; by default returns a
     * {@link JavaObjectLoader}.
     */
    protected JavaObjectLoader createObjectLoader() {
        return new JavaObjectLoader();
    }

    /**
     * Subclasses can override if required; by default returns a
     * {@link JavaDomainObjectContainer}.
     */
    protected JavaDomainObjectContainer createDomainObjectContainer() {
        return new JavaDomainObjectContainer();
    }
}
// Copyright (c) Naked Objects Group Ltd.
