package org.nakedobjects.nof.reflect.java.annotations;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.nakedobjects.noa.annotations.Annotation;


public abstract class AbstractAnnotationFactorySet implements AnnotationFactorySet {

    private Map factoriesByAnnotationType = new HashMap();

    public AnnotationFactory getFactory(Class annotationType) {
        return (AnnotationFactory) this.factoriesByAnnotationType.get(annotationType);
    }

    public void registerFactory(AnnotationFactory factory) {
        factoriesByAnnotationType.put(factory.getAnnotationType(), factory);
    }

    public Collection getFactories() {
        return factoriesByAnnotationType.values();
    }

    public Annotation[] process(Class clazz) {
        List annotations = new ArrayList();
        final Collection factories = getFactories();
        for (Iterator iter = factories.iterator(); iter.hasNext();) {
            AnnotationFactory factory = (AnnotationFactory) iter.next();
            final Annotation annotation = factory.process(clazz);
            if (annotation != null) {
                annotations.add(annotation);
            }
        }
        return (Annotation[]) annotations.toArray(new Annotation[] {});
    }

    public Annotation[] process(Method method) {
        List annotations = new ArrayList();
        final Collection factories = getFactories();
        for (Iterator iter = factories.iterator(); iter.hasNext();) {
            AnnotationFactory factory = (AnnotationFactory) iter.next();
            final Annotation annotation = factory.process(method);
            if (annotation != null) {
                annotations.add(annotation);
            }
        }
        return (Annotation[]) annotations.toArray(new Annotation[] {});
    }

    public Annotation[] processParams(Method method, int paramNum) {
        List annotations = new ArrayList();
        final Collection factories = getFactories();
        for (Iterator iter = factories.iterator(); iter.hasNext();) {
            AnnotationFactory factory = (AnnotationFactory) iter.next();
            Annotation annotation = factory.processParams(method, paramNum);
            if (annotation != null) {
                annotations.add(annotation);
            }
        }
        return (Annotation[]) annotations.toArray(new Annotation[] {});
    }

}
