package org.nakedobjects.nof.reflect.java.collection;

import org.nakedobjects.noa.adapter.NakedCollection;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.adapter.Persistable;
import org.nakedobjects.noa.adapter.ResolveState;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.core.adapter.AbstractNakedReference;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.core.util.ToString;


public abstract class AbstractCollectionAdapter extends AbstractNakedReference implements NakedCollection {
    private static final int INCOMPLETE_COLLECTION = -1;

    protected static NakedObject adapter(final Object element) {
        return NakedObjectsContext.getObjectLoader().getAdapterForElseCreateAdapterForTransient(element);
    }

    private NakedObjectSpecification elementSpecification;

    public AbstractCollectionAdapter(final NakedObjectSpecification spec) {
        elementSpecification = spec;
    }

    public NakedObjectSpecification getElementSpecification() {
        return elementSpecification;
    }
    
    public Persistable persistable() {
        return Persistable.TRANSIENT;
    }

    public String titleString() {
        int size = elementsLoaded() ? size(): INCOMPLETE_COLLECTION;
        if (elementSpecification == null || elementSpecification.getFullName().equals(Object.class.getName())) {
            switch (size) {
            case -1:
                return "Objects";
            case 0:
                return "No objects";
            case 1:
                return "1 object";
            default:
                return size + " objects";
            }
        } else {
            switch (size) {
            case -1:
                return elementSpecification.getPluralName();
            case 0:
                return "No " + elementSpecification.getPluralName();
            case 1:
                return "1 " + elementSpecification.getSingularName();
            default:
                return size + " " + elementSpecification.getPluralName();
            }
        }
    }

    public String toString() {
        ToString s = new ToString(this);
        toString(s);

        // title
        String title;
        try {
            title = "'" + this.titleString() + "'";
        } catch (NullPointerException e) {
            title = "none";
        }
        s.append("title", title);

        if (elementsLoaded()) {
            s.append("array", toStringElements());
        } else {
            s.append("elements", "not loaded yet");
        }

        return s.toString();
    }

    private boolean elementsLoaded() {
        ResolveState state = this.getResolveState();
        return state.isTransient() || state.isResolved();
    }

    abstract protected String toStringElements();
}
// Copyright (c) Naked Objects Group Ltd.
