package org.nakedobjects.nof.reflect.java.collection;

import java.util.ArrayList;

import org.nakedobjects.noa.adapter.NakedCollection;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.adapter.Persistable;
import org.nakedobjects.noa.adapter.ResolveState;
import org.nakedobjects.noa.reflect.Consent;
import org.nakedobjects.noa.reflect.NakedObjectField;
import org.nakedobjects.noa.spec.NakedCollectionSpecification;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.core.reflect.Veto;
import org.nakedobjects.nof.reflect.spec.AbstractSpecification;
import org.nakedobjects.nof.reflect.spec.ReflectionPeerBuilder;

public abstract class AbstractCollectionSpecification extends AbstractSpecification implements NakedCollectionSpecification {
    private static final int INCOMPLETE_COLLECTION = -1;

    protected static NakedObject adapter(final Object element) {
        return NakedObjectsContext.getObjectLoader().getAdapterForElseCreateAdapterForTransient(element);
    }

    private final  NakedObjectSpecification elementSpecification;

    public AbstractCollectionSpecification(String className, NakedObjectSpecification elementClass) {
        // to deal with generic collections remove the following prefix
        String prefix = "java.util.Arrays$";
        if (className.startsWith(prefix)) {
            this.className = "java.util." + className.substring(prefix.length());
        } else {
            this.className = className;
        }
        this.elementSpecification = elementClass;
    }

    public NakedObjectSpecification getElementType() {
        return elementSpecification;
    }
    
    public Persistable persistable() {
        return Persistable.TRANSIENT;
    }

    public String getTitle(NakedObject collection) {
        int size = elementsLoaded(collection) ? size(collection): INCOMPLETE_COLLECTION;
        if (elementSpecification == null || elementSpecification.getFullName().equals(Object.class.getName())) {
            switch (size) {
            case -1:
                return "Objects";
            case 0:
                return "No objects";
            case 1:
                return "1 object";
            default:
                return size + " objects";
            }
        } else {
            switch (size) {
            case -1:
                return elementSpecification.getPluralName();
            case 0:
                return "No " + elementSpecification.getPluralName();
            case 1:
                return "1 " + elementSpecification.getSingularName();
            default:
                return size + " " + elementSpecification.getPluralName();
            }
        }
    }

    private int size(NakedObject collection) {
        return ((NakedCollection) collection).size();
    }

    private boolean elementsLoaded(NakedObject collection) {
        ResolveState state = collection.getResolveState();
        return state.isTransient() || state.isResolved();
    }

    public int getType() {
        return COLLECTION;
    }

    public Consent isPersistable(NakedObject transientObject) {
        return Veto.DEFAULT;
    }

    public void introspect(ReflectionPeerBuilder builder) {
        fields = new NakedObjectField[0];
    }

    public NakedObjectField getField(String name) {
        return null;
    }

    public String getPluralName() {
        return "Collections";
    }

    public String getShortName() {
        return "Collection";
    }

    public String getSingularName() {
        return "Collection";
    }

    public String getDescription() {
        return "Collection";
    }

    public Object getDefaultValue() {
        return new ArrayList();
    }

}


// Copyright (c) Naked Objects Group Ltd.
