package org.nakedobjects.nof.reflect.java.collection;

import java.util.Collection;
import java.util.Enumeration;
import java.util.Iterator;

import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.spec.NakedCollectionSpecification;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.core.util.Assert;

public class JavaCollectionSpecification extends AbstractCollectionSpecification implements NakedCollectionSpecification {
    
    class ListEnumeration implements Enumeration {
        private final Iterator iterator;
        ListEnumeration(Iterator iterator) {
            this.iterator = iterator;
        }
        public boolean hasMoreElements() {
            return iterator.hasNext();
        }

        public Object nextElement() {
            Object element = iterator.next();
            return adapter(element);
        }
    }
    
    public JavaCollectionSpecification(final Class cls, final NakedObjectSpecification spec) {
        super(cls.getName(), spec);
    }

    public boolean contains(final NakedObject collection, final NakedObject object) {
        return collection(collection).contains(object.getObject());
    }

    public Enumeration elements(final NakedObject collection) {
        return new ListEnumeration(collection(collection).iterator());
    }
    
    public NakedObject firstElement(final NakedObject collection) {
        Collection javaCollection = collection(collection);
        if (javaCollection.size() == 0) {
            return null;
        } else {
            Object element = javaCollection.iterator().next();
            return adapter(element);
        }
    }

    public void init(final NakedObject collection, final Object[] initElements) {
        Collection javaCollection = collection(collection);
        Assert.assertEquals("Collection not empty", 0, javaCollection.size());
        for (int i = 0; i < initElements.length; i++) {
            javaCollection.add(initElements[i]);
        }
    }

    public int size(final NakedObject collection) {
        return collection(collection).size();
    }

    private Collection collection(final NakedObject collection) {
        return (Collection) collection.getObject();
    }
}


// Copyright (c) Naked Objects Group Ltd.
