package org.nakedobjects.nof.reflect.java.collection;

import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;

import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.core.util.Assert;


public class ListCollectionAdapter extends AbstractCollectionAdapter {
    private final List collection;

    class ListEnumeration implements Enumeration {
    	private final Iterator iterator;
    	ListEnumeration(Iterator iterator) {
    		this.iterator = iterator;
    	}
        public boolean hasMoreElements() {
            return iterator.hasNext();
        }

        public Object nextElement() {
            Object element = iterator.next();
            return adapter(element);
        }
    }
    
    public ListCollectionAdapter(final List list, final NakedObjectSpecification spec) {
        super(spec);
    	Assert.assertNotNull("List", list);
        Assert.assertNotNull("Element specification", spec);
        this.collection = list;
    }

    public boolean contains(final NakedObject object) {
        return collection.contains(object.getObject());
    }

    public Enumeration elements() {
    	return new ListEnumeration(collection.iterator());
    }

    public NakedObject firstElement() {
        if (size() == 0) {
            return null;
        } else {
            Object element = collection.get(0);
            return adapter(element);
        }
    }

    public Object getObject() {
        return collection;
    }

    public void init(final Object[] initElements) {
        Assert.assertEquals("Collection not empty", 0, this.collection.size());
        for (int i = 0; i < initElements.length; i++) {
            collection.add(initElements[i]);
        }
    }

    public int size() {
        return collection.size();
    }

    protected String toStringElements() {
    	return collection.toString();
    }
}
// Copyright (c) Naked Objects Group Ltd.
