package org.nakedobjects.nof.reflect.java.fixture;

import org.apache.log4j.Logger;
import org.nakedobjects.noa.NakedObjectRuntimeException;
import org.nakedobjects.nof.core.conf.Configuration;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.core.system.InstanceFactory;
import org.nakedobjects.nof.core.system.FixturesInstaller;
import org.nakedobjects.nof.core.util.NakedObjectConfiguration;


public class FixturesFromConfiguration implements FixturesInstaller {
    private static final Logger LOG = Logger.getLogger(FixturesFromConfiguration.class);
    private static final String NAKEDOBJECTS_FIXTURES = Configuration.ROOT + "fixtures";
    private static final String NAKEDOBJECTS_FIXTURES_PREFIX = Configuration.ROOT + "fixtures.prefix";

    public void installFixtures(final NakedObjectConfiguration configuration) {
        if (NakedObjectsContext.getObjectPersistor().isInitialized()) {
            LOG.info("skipping fixtures, as already loaded");
            return;
        }
        
        String fixturePrefix = configuration.getString(NAKEDOBJECTS_FIXTURES_PREFIX);
        fixturePrefix = fixturePrefix == null ? "" : fixturePrefix.trim();
        if (fixturePrefix.length() > 0 && !fixturePrefix.endsWith(Configuration.DELIMITER)) {
            fixturePrefix = fixturePrefix + Configuration.DELIMITER;
        }

        JavaFixtureBuilder builder = new JavaFixtureBuilder();
        String[] fixtureList = configuration.getList(NAKEDOBJECTS_FIXTURES);
        boolean fixtureLoaded = false;
        if (fixtureList != null) {
            for (int i = 0; i < fixtureList.length; i++) {
                String fixtureName = fixtureList[i];
                LOG.info("  running fixture " + fixtureName);
                Object fixture = InstanceFactory.createInstance(fixturePrefix + fixtureName, Object.class);
                fixtureLoaded = true;
                builder.addFixture(fixture);
            }
            builder.installFixtures();
            
            if (!fixtureLoaded) {
                throw new NakedObjectRuntimeException("Fixtures specified, but none loaded");
            }
        }
    }

    public String getName() {
        return "configuration-fixtures";
    }
}
// Copyright (c) Naked Objects Group Ltd.
