package org.nakedobjects.nof.reflect.java.fixture;

import java.util.Enumeration;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.nakedobjects.applib.fixtures.AbstractFixture;
import org.nakedobjects.noa.NakedObjectRuntimeException;
import org.nakedobjects.noa.adapter.NakedCollection;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.persist.NakedObjectPersistor;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.core.persist.TitleCriteria;


public abstract class InstancesFixture extends AbstractFixture {
    private static final Logger LOG = Logger.getLogger(InstancesFixture.class);
    private final Vector newInstances = new Vector();

    protected final Object createInstance(Class cls) {
        return createInstance(cls.getName());
    }

    protected void resetClock() {
        service.resetClock();
    }

    protected void setDate(final int year, final int month, final int day) {
        service.setDate(year, month, day);
    }

    protected void setTime(final int hour, final int minute) {
        service.setTime(hour, minute);
    }

    protected Object getInstance(final Class cls, final String title) {
        return findInstance(cls, title);
    }

    /**
     * Helper method to create an instance of the given type. Provided for exploration programs that need to
     * set up instances.
     */
    protected final Object createInstance(final String className) {
        NakedObjectSpecification spec = NakedObjectsContext.getReflector().loadSpecification(className);
        NakedObject object = NakedObjectsContext.getObjectLoader().createTransientInstance(spec);
        LOG.debug("adding " + object);
        newInstances.addElement(object);
        return object.getObject();
    }

    protected final Object findInstance(final Class cls, final String title) {
        Enumeration e = newInstances.elements();
        NakedObjectSpecification spec = NakedObjectsContext.getReflector().loadSpecification(cls);
        Vector instances = new Vector();
        while (e.hasMoreElements()) {
            NakedObject element = (NakedObject) e.nextElement();
            if (element.getSpecification() == spec && element.titleString().equals(title)) {
                instances.addElement(element);
            }
        }

        if (instances.size() > 1) {
            throw new NakedObjectRuntimeException("Found more than one instance of " + cls.getName() + " with title " + title
                    + ": " + instances.size());
        } else if (instances.size() == 1) {
            return ((NakedObject) instances.elementAt(0)).getObject();
        }

        NakedCollection collection = NakedObjectsContext.getObjectPersistor()
                .findInstances(new TitleCriteria(spec, title, false));
        if (collection.size() > 1) {
            throw new NakedObjectRuntimeException("Found more than one instance of " + cls.getName() + " with title " + title
                    + ": " + collection.size());
        } else if (collection.size() == 1) {
            return collection.firstElement().getObject();
        }

        throw new NakedObjectRuntimeException("Failed to find an instance of " + cls.getName() + " with title " + title);
    }

    private void saveNewObjects(final NakedObjectPersistor persistor) {
        for (int i = 0; i < newInstances.size(); i++) {
            NakedObject object = (NakedObject) newInstances.elementAt(i);

            boolean notPersistent = object.getResolveState().isTransient();
            if (notPersistent) {
                LOG.debug("persisting " + object);
                persistor.makePersistent(object);
            }
        }
        persistor.saveChanges();
        newInstances.removeAllElements();
    }

    public final void install() {
        installInstances();
        NakedObjectPersistor persistor = NakedObjectsContext.getObjectPersistor();
        saveNewObjects(persistor);
    }

    protected abstract void installInstances();

}
// Copyright (c) Naked Objects Group Ltd.
