package org.nakedobjects.nof.reflect.java.fixture;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.apache.log4j.Logger;
import org.nakedobjects.applib.fixtures.FixtureClock;
import org.nakedobjects.applib.fixtures.FixtureFactoryAndRepository;
import org.nakedobjects.applib.fixtures.FixtureServices;
import org.nakedobjects.noa.persist.NakedObjectPersistor;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.core.persist.AbstractFixtureBuilder;
import org.nakedobjects.nof.core.persist.FixtureException;


public class JavaFixtureBuilder extends AbstractFixtureBuilder {
    private static final Logger LOG = Logger.getLogger(JavaFixtureBuilder.class);
    private FixtureServices fixtureServices;
    private FixtureFactoryAndRepository fixtureRepository;

    protected void preInstallFixtures(NakedObjectPersistor objectManager) {
        fixtureRepository = new FixtureFactoryAndRepository();
        NakedObjectsContext.getObjectLoader().initDomainObject(fixtureRepository);
        FixtureClock clock = FixtureClock.initialize();
        fixtureServices = new JavaFixtureServices(clock);
    }
    
    protected void installFixture(Object fixture) {
        Method setMethod;
        try {
            setMethod = fixture.getClass().getMethod("setService", new Class[] { FixtureServices.class });
            LOG.debug("injecting fixtureServices into fixture");
            setMethod.invoke(fixture, new Object[] { fixtureServices });
        } catch (SecurityException e) {
            throw new FixtureException(e);
        } catch (NoSuchMethodException ignored) {
        } catch (IllegalArgumentException e) {
            throw new FixtureException(e);
        } catch (IllegalAccessException e) {
            throw new FixtureException("Unable to access setService method in " + fixture.getClass().getName(), e);
       } catch (InvocationTargetException e) {
           throw new FixtureException("Problem executing setService method in " + fixture.getClass().getName(), e);
       }
        
       try {
           setMethod = fixture.getClass().getMethod("setService", new Class[] { FixtureFactoryAndRepository.class });
           LOG.debug("injecting fixtureRepository into fixture");
           setMethod.invoke(fixture, new Object[] { fixtureRepository });
       } catch (SecurityException e) {
           throw new FixtureException(e);
       } catch (NoSuchMethodException ignored) {
       } catch (IllegalArgumentException e) {
           throw new FixtureException(e);
       } catch (IllegalAccessException e) {
           throw new FixtureException("Unable to access setService method in " + fixture.getClass().getName(), e);
      } catch (InvocationTargetException e) {
          throw new FixtureException("Problem executing setService method in " + fixture.getClass().getName(), e);
      }
      
        Method installMethod;
        try {
            installMethod = fixture.getClass().getMethod("install", new Class[0]);
            LOG.debug("invoking install method");
            installMethod.invoke(fixture, new Object[0]);
        } catch (SecurityException e) {
            throw new FixtureException(e);
        } catch (NoSuchMethodException e) {
//            throw new FixtureException("No install method found in " + fixture.getClass().getName(), e);
        } catch (IllegalArgumentException e) {
            throw new FixtureException(e);
        } catch (IllegalAccessException e) {
            throw new FixtureException("Unable to access install method in " + fixture.getClass().getName(), e);
       } catch (InvocationTargetException e) {
           throw new FixtureException("Problem executing install method in " + fixture.getClass().getName(), e);
       }
    }


    /**
     * Obtain any child fixtures for this fixture.
     * 
     * @param fixture
     */
    protected Object[] getFixtures(Object fixture) {
        Method getFixturesMethod;
        try {
            getFixturesMethod = fixture.getClass().getMethod("getFixtures", new Class[] {});
            LOG.debug("obtaining (any) child fixtures");
            Object[] childFixtures = (Object[]) getFixturesMethod.invoke(fixture, new Object[] {});
            return childFixtures;
        } catch (SecurityException e) {
            throw new FixtureException(e);
        } catch (NoSuchMethodException ignored) {
        } catch (ClassCastException ignored) {
        } catch (IllegalArgumentException e) {
            throw new FixtureException(e);
        } catch (IllegalAccessException e) {
            throw new FixtureException("Unable to access setService method in " + fixture.getClass().getName(), e);
       } catch (InvocationTargetException e) {
           throw new FixtureException("Problem executing setService method in " + fixture.getClass().getName(), e);
       }

       return new Object[]{};
    }
    
    
    
    
    
 
}
// Copyright (c) Naked Objects Group Ltd.
