package org.nakedobjects.nof.reflect.java.reflect;

import java.beans.Introspector;
import java.lang.reflect.Array;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.nakedobjects.applib.DomainObjectContainer;
import org.nakedobjects.applib.marker.AlwaysImmutable;
import org.nakedobjects.applib.marker.Bounded;
import org.nakedobjects.applib.marker.ImmutableOncePersisted;
import org.nakedobjects.applib.marker.NonPersistable;
import org.nakedobjects.applib.marker.ProgramPersistable;
import org.nakedobjects.applib.security.UserMemento;
import org.nakedobjects.applib.value.MultilineString;
import org.nakedobjects.noa.NakedObjectRuntimeException;
import org.nakedobjects.noa.adapter.FieldRegEx;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.adapter.Persistable;
import org.nakedobjects.noa.annotations.Annotation;
import org.nakedobjects.noa.annotations.ValueAnnotation;
import org.nakedobjects.noa.annotations.When;
import org.nakedobjects.noa.reflect.NakedObjectAction;
import org.nakedobjects.noa.spec.HasLifecycle;
import org.nakedobjects.noa.spec.HasType;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.noa.spec.NakedObjectSpecificationException;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.core.util.NameConvertor;
import org.nakedobjects.nof.reflect.java.annotations.AnnotationFactorySet;
import org.nakedobjects.nof.reflect.java.annotations.DefaultAnnotationFactorySet;
import org.nakedobjects.nof.reflect.java.annotations.JavaIntrospectorAnnotationExtension;
import org.nakedobjects.nof.reflect.peer.ActionPeer;
import org.nakedobjects.nof.reflect.peer.FieldPeer;
import org.nakedobjects.nof.reflect.peer.MemberIdentifier;
import org.nakedobjects.nof.reflect.peer.MemberIdentifierImpl;
import org.nakedobjects.nof.reflect.peer.MemberPeer;
import org.nakedobjects.nof.reflect.peer.ObjectTitle;
import org.nakedobjects.nof.reflect.peer.ReflectionException;
import org.nakedobjects.nof.reflect.spec.AbstractSpecification;
import org.nakedobjects.nof.reflect.spec.CollectionSpecification;
import org.nakedobjects.nof.reflect.spec.ReflectionPeerBuilder;


class JavaIntrospector {
    private static final String AUTHORISED_PREFIX = "hide";
    private static final String SESSION_HIDE_PREFIX = "hide";
    private static final String SESSION_READONLY_PREFIX = "disable";
    private static final String UNUSABLE_PREFIX = "disable";
    protected static final boolean CLASS = true;
    private static final String DESCRIPTION_PREFIX = "description";
    protected static final String GET_PREFIX = "get";
    private static final String HELP_PREFIX = "help";
    private static final String HIDDEN_PREFIX = "alwaysHide";
    private static final Logger LOG = Logger.getLogger(JavaIntrospector.class);
    private static final String OPTIONAL_PREFIX = "optional";
    private static final String PROTECTED_PREFIX = "protect";
    private static final String NAME_PREFIX = "name";
    private static final Object[] NO_PARAMETERS = new Object[0];
    private static final Class[] NO_PARAMETERS_TYPES = new Class[0];
    protected static final boolean OBJECT = false;
    protected static final String SET_PREFIX = "set";
    protected static final String DOTNET_SET_PREFIX = "set_";
    protected static final String DEFAULT_PREFIX = "default";
    protected static final String OPTIONS_PREFIX = "choices";
    protected static final String CLEAR_PREFIX = "clear";
    protected static final String MODIFY_PREFIX = "modify";
    protected static final String VALID_PREFIX = "validate";
    private static final String HIDE_PREFIX = "hide";
    private static final String PARAMETER_NAMES_PREFIX = "names";
    private static final String PARAMETER_DESCRIPTIONS_PREFIX = "descriptions";
    private static final String PARAMETER_OPTIONAL_PREFIX = "optional";
    private static final String PARAMETER_DEFAULTS_PREFIX = "default";
    private static final String PARAMETER_OPTIONS_PREFIX = "choices";
    private static final String[] objectMethodNames;
    private static final Class[] objectMethodTypes;
    private static final Class[][] objectMethodParameters;

    /*
     * .Net fails to find methods properly for root class, so we use a saved set.
     */
    static {
        Method[] methods = Object.class.getMethods();
        int size = methods.length;
        objectMethodNames = new String[size];
        objectMethodTypes = new Class[size];
        objectMethodParameters = new Class[size][];
        for (int i = 0; i < methods.length; i++) {
            Method method = methods[i];
            objectMethodNames[i] = method.getName();
            objectMethodTypes[i] = method.getReturnType();
            objectMethodParameters[i] = method.getParameterTypes();
        }
    }

    /**
     * Returns the name of a Java entity without any prefix. A prefix is defined as the first set of lowercase
     * letters and the name is characters from, and including, the first upper case letter. If no upper case
     * letter is found then an empty string is returned.
     * 
     * <p>
     * Calling this method with the following Java names will produce these results:
     * 
     * <pre>
     *                     getCarRegistration        -&gt; CarRegistration
     *                     CityMayor -&gt; CityMayor
     *                     isReady -&gt; Ready
     * </pre>
     * 
     */
    static String javaBaseName(String javaName) {
        int pos = 0;

        // find first upper case character
        int len = javaName.length();

        while ((pos < len) && (javaName.charAt(pos) != '_') && Character.isLowerCase(javaName.charAt(pos))) {
            pos++;
        }

        if (pos >= len) {
            return "";
        }

        if (javaName.charAt(pos) == '_') {
            pos++;
        }

        if (pos >= len) {
            return "";
        }

        String baseName = javaName.substring(pos);
        char firstChar = baseName.charAt(0);

        if (Character.isLowerCase(firstChar)) {
            return Character.toUpperCase(firstChar) + baseName.substring(1);
        } else {
            return baseName;
        }
    }

    private String className;
    private Method clearDirtyMethod;
    private Class cls;
    private Method isDirtyMethod;
    private Method markDirtyMethod;
    private Method methods[];
    private final JavaReflector reflector;
    private ObjectTitle title;
    private Method iconMethod;
    private JavaIntrospectorAnnotationExtension extension;
    private Method pluralNameMethod;
    private Method singularNameMethod;
    private Method[] lifecycleMethods;
    private Method canSaveMethod;
    private boolean isEntity;
    private int type;
    private OrderSet orderedFields;
    private OrderSet orderedObjectActions;
    private OrderSet orderedClassActions;
    private final AbstractSpecification specification;
    private final ExtensionHolderMutable extensionHolder;

    private AnnotationFactorySet annotationFactorySet = new DefaultAnnotationFactorySet();

    private Object[] processAnnotations(Class clazz) {
        return annotationFactorySet.process(clazz);
    }

    private Object[] processAnnotations(Method method) {
        return annotationFactorySet.process(method);
    }

    private Object[] processParameterAnnotations(Method method, int paramNum) {
        return annotationFactorySet.processParams(method, paramNum);
    }

    JavaIntrospector(
            final Class cls,
            final AbstractSpecification specification,
            final ReflectionPeerBuilder builder,
            final JavaReflector reflector,
            ExtensionHolderMutable extensionHolder) {
        this.specification = specification;
        // private interface into JavaSpecification, to allow annotations to be added.
        this.extensionHolder = extensionHolder;
        LOG.debug("creating JavaIntrospector for " + cls);

        this.reflector = reflector;

        if (!Modifier.isPublic(cls.getModifiers())) {
            throw new NakedObjectSpecificationException("An adapted class must be marked as public.  Error in " + cls);
        }
        this.cls = cls;
        methods = cls.getMethods();
        className = cls.getName();

        extension = new JavaIntrospectorAnnotationExtension(cls);
    }

    private boolean isSynthetic(final Method method) {
        try {
            final Class cls = method.getClass();
            try {
                return ((Boolean) cls.getMethod("isSynthetic", (Class[]) null).invoke(method, (Object[]) null)).booleanValue();
            } catch (final NoSuchMethodException nsm) {
                // pre java 5 
                return false;
            }
        } catch (final Exception e) {
            throw new NakedObjectRuntimeException(e);
        }
    }

    private ActionPeer[] findActionMethods(final boolean forClass) {
        LOG.debug("  looking for action methods");
        Vector actions = new Vector();
        for (int i = 0; i < methods.length; i++) {
            if (methods[i] == null) {
                continue;
            }
            Method method = methods[i];
            if (Modifier.isStatic(method.getModifiers()) != forClass) {
                continue;
            }

            String fullMethodName = method.getName();
            // TODO this is not a great way to do this, need to improve it
            if (startsWith(fullMethodName, AUTHORISED_PREFIX) || startsWith(fullMethodName, OPTIONAL_PREFIX)
                    || startsWith(fullMethodName, PROTECTED_PREFIX) || startsWith(fullMethodName, PARAMETER_NAMES_PREFIX)
                    || startsWith(fullMethodName, PARAMETER_OPTIONS_PREFIX) || startsWith(fullMethodName, MODIFY_PREFIX)
                    || startsWith(fullMethodName, PARAMETER_OPTIONAL_PREFIX) || startsWith(fullMethodName, DEFAULT_PREFIX)
                    || startsWith(fullMethodName, OPTIONS_PREFIX) || startsWith(fullMethodName, PARAMETER_DEFAULTS_PREFIX)
                    || startsWith(fullMethodName, VALID_PREFIX) || startsWith(fullMethodName, HIDE_PREFIX)
                    || startsWith(fullMethodName, UNUSABLE_PREFIX) || startsWith(fullMethodName, NAME_PREFIX)
                    || startsWith(fullMethodName, DESCRIPTION_PREFIX) || startsWith(fullMethodName, SET_PREFIX)
                    || startsWith(fullMethodName, GET_PREFIX) || startsWith(fullMethodName, HELP_PREFIX) 
                    || startsWith(fullMethodName, DOTNET_SET_PREFIX)) {
                continue;
            }

            if (method.toString().equals("public abstract java.util.Iterator java.lang.Iterable.iterator()")) {
                LOG.debug("ignoring " + method);
                continue;
            }
            if (isSynthetic(method)) {
                LOG.debug("ignoring (synthetic) " + method);
                continue;
            }

            LOG.debug("  identified action " + method);
            methods[i] = null;

            String capitalizedName = fullMethodName.substring(0, 1).toUpperCase() + fullMethodName.substring(1);

            NakedObjectAction.Type type = NakedObjectAction.USER;
            if (extension.isDebug(method)) {
                type = NakedObjectAction.DEBUG;
            } else if (capitalizedName.startsWith("Debug")) {
                type = NakedObjectAction.DEBUG;
                capitalizedName = capitalizedName.substring(5);
            } else if (extension.isExploration(method)) {
                type = NakedObjectAction.EXPLORATION;
            } else if (capitalizedName.startsWith("Exploration")) {
                type = NakedObjectAction.EXPLORATION;
                capitalizedName = capitalizedName.substring(11);
            }

            NakedObjectAction.Target target = NakedObjectAction.DEFAULT;
            if (extension.isLocal(method)) {
                target = NakedObjectAction.LOCAL;
            } else if (capitalizedName.startsWith("Local")) {
                target = NakedObjectAction.LOCAL;
                capitalizedName = capitalizedName.substring(5);
            } else if (extension.isRemote(method)) {
                target = NakedObjectAction.REMOTE;
            } else if (capitalizedName.startsWith("Remote")) {
                target = NakedObjectAction.REMOTE;
                capitalizedName = capitalizedName.substring(6);
            }

            Class[] params = method.getParameterTypes();

            Method validMethod = findMethod(forClass, JavaIntrospector.VALID_PREFIX + capitalizedName, String.class, params);

            MethodPattern methodPattern = new MethodPattern(method, capitalizedName, params);
            DescriptiveMethods descriptiveMethods = findDescriptiveMethods(methodPattern);
            ParameterMethods parameterMethods = findActionParameterMethods(forClass, methodPattern);
            GeneralControlMethods controlMethods = findControlMethods(forClass, methodPattern, validMethod, null);

            Class[] sessionParams = new Class[params.length + 1];
            sessionParams[0] = UserMemento.class;
            System.arraycopy(params, 0, sessionParams, 1, params.length);

            MemberSessionMethods sessionMethods = findSessionMethods(capitalizedName, params);

            if (isMemberHidden(capitalizedName, method, params) == When.ALWAYS) {
                // methods that are hidden, should be ignored
                continue;
            }

            ActionFlags actionFlags = setActionFlags(capitalizedName, method, params);
            ExtensionHolderMutableActionPeer local = createActionPeer(Introspector.decapitalize(capitalizedName), type, target,
                    specification, method, parameterMethods, descriptiveMethods, controlMethods, actionFlags, sessionMethods);
            final Annotation[] annotations = annotationFactorySet.process(method);
            AnnotationUtil.addAnnotations(local, annotations);
            actions.addElement(local);
        }

        return convertToArray(actions);
    }

    private boolean startsWith(final String fullMethodName, final String prefix) {
        int length = prefix.length();
        if (length >= fullMethodName.length()) {
            return false;
        } else {
            char startingCharacter = fullMethodName.charAt(length);
            return fullMethodName.startsWith(prefix) && Character.isUpperCase(startingCharacter);
        }
    }

    String className() {
        return className;
    }

    /**
     * Recursively move up the class hierarchy until a Java root class is found e.g. Vector, Object etc, and
     * remove all method associated with that type.
     */
    private void clearSuperclassMethods(final Class cls) {
        if (cls == null) {
            return;
        }

        if (cls.getName().startsWith("java.")) {
            if (cls.getName().startsWith("java.lang.Object")) {
                /* .Net fails to find methods properly for root class, so we used the saved set. */
                for (int i = 0; i < objectMethodNames.length; i++) {
                    removeMethod(false, objectMethodNames[i], objectMethodTypes[i], objectMethodParameters[i]);
                }
            } else {
                Method[] methods = cls.getMethods();
                for (int j = 0; j < methods.length; j++) {
                    Method method = methods[j];
                    removeMethod(false, method.getName(), method.getReturnType(), method.getParameterTypes());
                }
            }
        } else {
            clearSuperclassMethods(cls.getSuperclass());
        }

    }

    private void clearUnwantedMethods() {
        // setContainer is for the app lib
        removeMethod(false, "setContainer", void.class, new Class[] { DomainObjectContainer.class });
        removeMethod(false, "set_Container", void.class, new Class[] { DomainObjectContainer.class });
        removeMethod(false, "init", void.class, new Class[0]);

        // static get/set methods
        findPrefixedMethods(true, "get", null, false, 0);
        findPrefixedMethods(true, "set", null, false, 0);

    }

    private ActionPeer[] convertToArray(final Vector actions) {
        ActionPeer results[] = new ActionPeer[actions.size()];
        Enumeration actionEnumeration = actions.elements();
        int i = 0;
        while (actionEnumeration.hasMoreElements()) {
            results[i++] = (ActionPeer) actionEnumeration.nextElement();

        }
        return (ActionPeer[]) results;
    }

    private ExtensionHolderMutableActionPeer createActionPeer(
            final String name,
            final NakedObjectAction.Type action,
            final NakedObjectAction.Target target,
            final NakedObjectSpecification targetx,
            final Method method,
            final ParameterMethods parameterMethods,
            final DescriptiveMethods descriptiveMethods,
            final GeneralControlMethods controlMethods,
            final ActionFlags actionFlags,
            final MemberSessionMethods sessionMethods) {
        Class[] cls = method.getParameterTypes();
        int noParameters = cls.length;
        NakedObjectSpecification[] parameters = new NakedObjectSpecification[noParameters];
        for (int i = 0; i < parameters.length; i++) {
            parameters[i] = specification(cls[i]);
        }
        MemberIdentifier identifier = new MemberIdentifierImpl(className, name, cls);
        Class elementType = extension.getCollectionType(method);
        NakedObjectSpecification returnType;
        if (elementType != null) {
            returnType = new CollectionSpecification(method.getReturnType().getName(), elementType);
        } else if (method.getReturnType() == void.class) {
            returnType = null;
        } else {
            returnType = NakedObjectsContext.getReflector().loadSpecification(method.getReturnType());
        }

        final JavaAction javaAction = new JavaAction(identifier, action, target, targetx, parameters, method, returnType,
                parameterMethods, descriptiveMethods, controlMethods, actionFlags, sessionMethods, annotationFactorySet);

        final int numberParameters = method.getParameterTypes().length;
        for (int i = 0; i < numberParameters; i++) {
            final Annotation[] paramAnnotations = annotationFactorySet.processParams(method, i);
        }

        return javaAction;
    }

    private OrderSet createOrderSet(final String order, final MemberPeer[] members) {
        if (order != null) {
            OrderSet set;
            set = SimpleOrderSet.createOrderSet(order, members);
            return set;
        } else {
            OrderSet set = DeweyOrderSet.createOrderSet(members);
            return set;
        }
    }

    private FieldPeer[] findFieldMethods() {
        if (cls.getName().startsWith("java.") || getClassStrategy().isSystemClass(cls)) {
            LOG.debug("skipping fields in " + cls.getName());
            return new FieldPeer[0];
        }
        LOG.debug("  looking for fields for " + cls);

        removeMethod(false, "getClass", Class.class, null);
        Class[] invalidFieldTypes = getClassStrategy().getInvalidFieldTypes();
        if (invalidFieldTypes != null) {
            for (int i = 0; i < invalidFieldTypes.length; i++) {
                removeFields(invalidFieldTypes[i]);
            }
        }

        Vector elements = new Vector();

        // ensure all return types are known; in case any are value types.
        Vector getterCandidates = new Vector();
        locateCandidateGetters("is", getterCandidates);
        locateCandidateGetters("get", getterCandidates);
        for (Iterator iter = getterCandidates.iterator(); iter.hasNext();) {
            Method getterCandidate = (Method) iter.next();
            final Class returnType = getterCandidate.getReturnType();
            if (!void.class.equals(returnType) && returnType != cls) {
                reflector.loadSpecification(returnType);
            }
        }

        Class[] valueTypes = getClassStrategy().getValueTypes();
        for (int i = 0; i < valueTypes.length; i++) {
            if (valueTypes[i].equals(boolean.class)) {
                findValueFields(elements, valueTypes[i], "is");
            }
            findValueFields(elements, valueTypes[i], GET_PREFIX);
        }

        // need to find one-many first, so they are not mistaken as one-one associations
        OneToManyStrategy[] oneToManyStrategies = reflector.getOneToManyStrategies();
        for (int i = 0; i < oneToManyStrategies.length; i++) {
            oneToManyStrategies[i].introspectAssociations(this, extension, annotationFactorySet, elements);
        }
        findOneToOneAssociationFields(elements);

        FieldPeer[] results = new FieldPeer[elements.size()];
        elements.copyInto(results);
        return results;
    }

    protected void finalize() throws Throwable {
        super.finalize();
        LOG.debug("finalizing reflector " + this);
    }

    GeneralControlMethods findControlMethods(boolean type, MethodPattern methodPattern, Method validMethod1, Method validMethod2) {
        String name = methodPattern.getName();
        Class[] parameters = methodPattern.getParameterTypes();
        Method method = methodPattern.getMethod();
        MemberHelper hidden;
        hidden = findMethod(type, boolean.class, HIDE_PREFIX + name, parameters, null);
        MemberHelper unusable;
        // TODO is the use of the extension correct here, presumably we need to do this for static methods
        unusable = findMethod(type, String.class, UNUSABLE_PREFIX + name, parameters, extension.isUnusable(method));
        GeneralControlMethods controlMethods = new GeneralControlMethods(hidden, unusable, validMethod1, validMethod2);
        return controlMethods;
    }

    ParameterMethods findActionParameterMethods(boolean type, MethodPattern methodPattern) {
        Class[] parameters = methodPattern.getParameterTypes();
        MemberHelper names;
        MemberHelper descriptions;
        MemberHelper optional;
        MemberHelper defaults;
        MemberHelper options;
        int[] maxLengths;
        int[] typicalLengths;
        int[] noLines;
        boolean[] canWrap;
        if (parameters.length == 0) {
            NoMemberHelper noMemberHelper = new NoMemberHelper();
            names = noMemberHelper;
            descriptions = noMemberHelper;
            optional = noMemberHelper;
            defaults = noMemberHelper;
            options = noMemberHelper;
            maxLengths = new int[0];
            typicalLengths = new int[0];
            noLines = new int[0];
            canWrap = new boolean[0];
        } else {
            String name = methodPattern.getName();
            Method method = methodPattern.getMethod();
            names = findMethod(CLASS, String[].class, PARAMETER_NAMES_PREFIX + name, parameters, extension
                    .getParameterNames(method));
            descriptions = findMethod(CLASS, String[].class, PARAMETER_DESCRIPTIONS_PREFIX + name, parameters, extension
                    .getParameterDescriptions(method));
            optional = findMethod(CLASS, boolean[].class, PARAMETER_OPTIONAL_PREFIX + name, parameters, extension
                    .getOptionalParameters(method));

            boolean allParametersSameType = true;
            for (int i = 1; i < parameters.length; i++) {
                if (parameters[i] != parameters[0]) {
                    allParametersSameType = false;
                    break;
                }
            }

            defaults = findMethod(type, Object[].class, PARAMETER_DEFAULTS_PREFIX + name, parameters, extension
                    .getParameterDefaults(method));
            // it is valid to use the proper type in the return if all parameters are the same type
            if (defaults instanceof NoMemberHelper) {
                if (allParametersSameType) {
                    Object arrayClass = Array.newInstance(parameters[0], 0);
                    defaults = findMethod(type, arrayClass.getClass(), PARAMETER_DEFAULTS_PREFIX + name, parameters, null);
                }
            }

            options = findMethod(type, Object[].class, PARAMETER_OPTIONS_PREFIX + name, parameters, extension
                    .getParameterOptions(method));
            if (options instanceof NoMemberHelper) {
                if (allParametersSameType) {
                    Object arrayClass = Array.newInstance(parameters[0], new int[] { 0, 0 });
                    options = findMethod(type, arrayClass.getClass(), PARAMETER_OPTIONS_PREFIX + name, parameters, null);
                }
                /*
                 * if (options == null) { options = new NoMemberHelper(); }
                 */
            }

            maxLengths = extension.getParametersMaxLengths(method);
            typicalLengths = extension.getParametersTypicalLengths(method);
            noLines = extension.getParametersNoLines(method);
            for (int i = 1; i < parameters.length; i++) {
                if (noLines[i] == 1 && parameters[i].isAssignableFrom(MultilineString.class)) {
                    noLines[i] = 6;
                }
            }
            canWrap = extension.getParametersCanWrap(method);

        }
        ParameterMethods controlMethods = new ParameterMethods(names, descriptions, optional, defaults, options, maxLengths,
                typicalLengths, noLines, canWrap);
        return controlMethods;
    }

    DescriptiveMethods findDescriptiveMethods(MethodPattern methodPattern) {
        String methodName = methodPattern.getName();
        Class[] parameters = methodPattern.getParameterTypes();
        Method method = methodPattern.getMethod();
        MemberHelper name;
        name = findMethod(CLASS, String.class, NAME_PREFIX + methodName, parameters, extension.getMemberName(method));
        MemberHelper description;
        description = findMethod(CLASS, String.class, DESCRIPTION_PREFIX + methodName, parameters, extension
                .getMemberDescription(method));
        DescriptiveMethods descriptiveMethods = new DescriptiveMethods(name, description);
        return descriptiveMethods;
    }

    private MemberHelper findMethod(boolean forClass, Class returnType, String name, Class[] parameters, MemberHelper helper) {
        if (helper == null) {
            Method method = findMethod(forClass, name, returnType, parameters);
            if (method == null) {
                method = findMethod(forClass, name, returnType, NO_PARAMETERS_TYPES);
            }
            if (method != null) {
                helper = new MethodHelper(method);
            }
            if (method == null) {
                helper = new NoMemberHelper();
            }
        }
        return helper;
    }

    /**
     * Returns a specific public methods that: have the specified prefix; have the specified return type, or
     * void, if canBeVoid is true; and has the specified number of parameters. If the returnType is specified
     * as null then the return type is ignored.
     * 
     * @param forClass
     * @param name
     * @param returnType
     * @param paramTypes
     *            the set of parameters the method should have, if null then is ignored
     * @return Method
     */
    Method findMethod(final boolean forClass, final String name, final Class returnType, final Class[] paramTypes) {
        method: for (int i = 0; i < methods.length; i++) {
            if (methods[i] == null) {
                continue;
            }

            Method method = methods[i];
            int modifiers = method.getModifiers();

            // check for public modifier
            if (!Modifier.isPublic(modifiers)) {
                continue;
            }

            // check for static modifier
            if (Modifier.isStatic(modifiers) != forClass) {
                continue;
            }

            // check for name
            if (!method.getName().equals(name)) {
                continue;
            }

            // check for return type
            if (returnType != null && returnType != method.getReturnType()) {
                continue;
            }

            // check params (if required)
            if (paramTypes != null) {
                Class[] parameterTypes = method.getParameterTypes();
                if (paramTypes.length != parameterTypes.length) {
                    continue;
                }

                for (int c = 0; c < paramTypes.length; c++) {
                    if ((paramTypes[c] != null) && (paramTypes[c] != parameterTypes[c])) {
                        continue method;
                    }
                }
            }
            methods[i] = null;

            return method;
        }

        return null;
    }

    /**
     * Returns a Vector of public methods that: have the specified prefix; have the specified return type, or
     * void, if canBeVoid is true; and has the specified number of parameters. If the returnType is specified
     * as null then the return type is ignored.
     */
    Vector findPrefixedMethods(
            final boolean forClass,
            final String prefix,
            final Class returnType,
            final boolean canBeVoid,
            final int paramCount) {
        Vector validMethods = new Vector();

        for (int i = 0; i < methods.length; i++) {
            if (methods[i] == null) {
                continue;
            }

            Method method = methods[i];

            if (Modifier.isStatic(method.getModifiers()) != forClass) {
                continue;
            }

            boolean goodPrefix = method.getName().startsWith(prefix);

            boolean goodCount = method.getParameterTypes().length == paramCount;
            Class type = method.getReturnType();
            boolean goodReturn = ((returnType == null) || (canBeVoid && (type == void.class)) || returnType
                    .isAssignableFrom(type))
                    && !getClassStrategy().isSystemClass(type);

            if (goodPrefix && goodCount && goodReturn) {
                validMethods.addElement(method);
                methods[i] = null;
            }
        }
        return validMethods;
    }

    void locateCandidateGetters(final String prefix, final Vector candidates) {

        for (int i = 0; i < methods.length; i++) {
            if (methods[i] == null) {
                continue;
            }

            Method method = methods[i];

            if (Modifier.isStatic(method.getModifiers())) {
                continue;
            }

            boolean goodPrefix = method.getName().startsWith(prefix);

            boolean goodCount = method.getParameterTypes().length == 0;

            if (goodPrefix && goodCount) {
                candidates.add(method);
            }
        }
    }

    Vector findPrefixedMethods(final boolean forClass, final String prefix, final Class returnType, final int paramCount) {
        return findPrefixedMethods(forClass, prefix, returnType, false, paramCount);
    }

    OrderSet getClassActions() {
        return orderedClassActions;
    }

    Method getClearDirtyMethod() {
        return clearDirtyMethod;
    }

    public OrderSet getFields() {
        return orderedFields;
    }

    String getFullName() {
        return cls.getName();
    }

    String[] getInterfaces() {
        Class[] interfaces = cls.getInterfaces();
        Class[] nakedInterfaces = new Class[interfaces.length];
        int validInterfaces = 0;
        for (int i = 0; i < interfaces.length; i++) {
            nakedInterfaces[validInterfaces++] = interfaces[i];
        }

        String[] interfaceNames = new String[validInterfaces];
        for (int i = 0; i < validInterfaces; i++) {
            interfaceNames[i] = nakedInterfaces[i].getName();
        }

        return interfaceNames;
    }

    Method getIsDirtyMethod() {
        return isDirtyMethod;
    }

    Method getMarkDirtyMethod() {
        return markDirtyMethod;
    }

    OrderSet getObjectActions() {
        return orderedObjectActions;
    }

    String getSuperclass() {
        Class superclass = cls.getSuperclass();

        if (superclass == null) {
            return null;
        }
        return superclass.getName();
    }

    int getType() {
        return type;
    }

    /**
     * We need to do this early so we call in the JavaSpecification's constructor.
     */
    void addAnnotationsAsExtensions() {
        if (extensionHolder != null) {
            Annotation[] annotations = annotationFactorySet.process(cls);
            AnnotationUtil.addAnnotations(extensionHolder, annotations);
        }
    }

    void introspect() {
        LOG.info("introspecting " + cls.getName());

        titleMethod();

        clearSuperclassMethods(cls);
        clearUnwantedMethods();

        pluralNameMethod = findMethod(CLASS, "pluralName", String.class, NO_PARAMETERS_TYPES);
        singularNameMethod = findMethod(CLASS, "singularName", String.class, NO_PARAMETERS_TYPES);

        isDirtyMethod = findMethod(OBJECT, "isDirty", boolean.class, NO_PARAMETERS_TYPES);
        clearDirtyMethod = findMethod(OBJECT, "clearDirty", void.class, NO_PARAMETERS_TYPES);
        markDirtyMethod = findMethod(OBJECT, "markDirty", void.class, NO_PARAMETERS_TYPES);

        iconMethod = findMethod(OBJECT, "iconName", String.class, NO_PARAMETERS_TYPES);

        lifecycleMethods = new Method[9];
        lifecycleMethods[HasLifecycle.CREATED] = findMethod(OBJECT, "created", void.class, NO_PARAMETERS_TYPES);
        lifecycleMethods[HasLifecycle.SAVING] = findMethod(OBJECT, "saving", void.class, NO_PARAMETERS_TYPES);
        lifecycleMethods[HasLifecycle.SAVED] = findMethod(OBJECT, "saved", void.class, NO_PARAMETERS_TYPES);
        lifecycleMethods[HasLifecycle.UPDATING] = findMethod(OBJECT, "updating", void.class, NO_PARAMETERS_TYPES);
        lifecycleMethods[HasLifecycle.UPDATED] = findMethod(OBJECT, "updated", void.class, NO_PARAMETERS_TYPES);
        lifecycleMethods[HasLifecycle.LOADING] = findMethod(OBJECT, "loading", void.class, NO_PARAMETERS_TYPES);
        lifecycleMethods[HasLifecycle.LOADED] = findMethod(OBJECT, "loaded", void.class, NO_PARAMETERS_TYPES);
        lifecycleMethods[HasLifecycle.DELETING] = findMethod(OBJECT, "deleting", void.class, NO_PARAMETERS_TYPES);
        lifecycleMethods[HasLifecycle.DELETED] = findMethod(OBJECT, "deleted", void.class, NO_PARAMETERS_TYPES);

        canSaveMethod = findMethod(OBJECT, "validate", String.class, NO_PARAMETERS_TYPES);

        isEntity = extension.isEntity();

        boolean hasValueAnnotation = false;
        if (extensionHolder != null) {
            ValueAnnotation valueAnnotation = (ValueAnnotation) extensionHolder.getExtension(ValueAnnotation.class);
            hasValueAnnotation = (valueAnnotation != null);
            if (hasValueAnnotation) {
                getClassStrategy().addValueType(cls);
                Class valueAdapterClass = valueAnnotation.adapterType();
                if (valueAdapterClass == null) {
                    throw new NakedObjectSpecificationException("No value adapter found: " + valueAnnotation.adapterTypeName());
                }
                NakedObjectsContext.getObjectLoader().addAdapterClass(cls, valueAdapterClass);
                hasValueAnnotation = true;
            }
        }

        type = HasType.OBJECT;
        if (hasValueAnnotation || isValueType()) {
            type = HasType.VALUE;
        } else {
            OneToManyStrategy[] oneToManyStrategies = reflector.getOneToManyStrategies();
            for (int i = 0; i < oneToManyStrategies.length; i++) {
                if (oneToManyStrategies[i].getReturnType().isAssignableFrom(cls)) {
                    type = HasType.COLLECTION;
                }
            }
        }

        String fieldOrder = extension.getFieldOrder();
        if (fieldOrder == null) {
            fieldOrder = invokeSortOrderMethod("field");
        }
        final FieldPeer[] findFieldMethods = findFieldMethods();
        orderedFields = createOrderSet(fieldOrder, findFieldMethods);

        String actionOrder = extension.getObjectActionOrder();
        if (actionOrder == null) {
            actionOrder = invokeSortOrderMethod("action");
        }
        final ActionPeer[] findObjectActionMethods = findActionMethods(OBJECT);
        orderedObjectActions = createOrderSet(actionOrder, findObjectActionMethods);

        actionOrder = extension.getClassActionOrder();
        if (actionOrder == null) {
            actionOrder = invokeSortOrderMethod("classAction");
        }
        final ActionPeer[] findClassActionMethods = findActionMethods(CLASS);
        orderedClassActions = createOrderSet(actionOrder, findClassActionMethods);
    }

    private boolean isValueType() {
        if (getClassStrategy().isValueType(cls)) {
            return true;
        }

        Class[] valueTypes = reflector.getClassStrategy().getValueTypes();
        for (int i = 0; i < valueTypes.length; i++) {
            if (valueTypes[i].isAssignableFrom(cls)) {
                return true;
            }
        }

        return false;
    }

    private ClassStrategy getClassStrategy() {
        return reflector.getClassStrategy();
    }

    boolean isAbstract() {
        return Modifier.isAbstract(cls.getModifiers());
    }

    public boolean isEntity() {
        return isEntity;
    }

    boolean isExtendable() {
        return Modifier.isFinal(cls.getModifiers());
    }

    boolean isAggregated() {
        return false; // extension.isAggregated() || Aggregated.class.isAssignableFrom(cls);
    }

    boolean isBounded() {
        return extension.isBounded() || Bounded.class.isAssignableFrom(cls);
    }

    boolean isSearchable() {
        return extension.isSearchable();
    }

    public boolean isImmutableOncePersisted() {
        return extension.isImmutableOncePersisted() || ImmutableOncePersisted.class.isAssignableFrom(cls);
    }

    public boolean isAlwaysImmutable() {
        return extension.isAlwaysImmutable() || AlwaysImmutable.class.isAssignableFrom(cls);
    }

    public boolean isCached() {
        return false; /* extension.isCached() || Cached.class.isAssignableFrom(cls); */
    }

    /**
     * Returns a vector of Association fields for all the get methods that use NakedObjects.
     * 
     * @throws ReflectionException
     */
    private void findOneToOneAssociationFields(final Vector associations) throws ReflectionException {
        Vector v = findPrefixedMethods(OBJECT, GET_PREFIX, Object.class, 0);

        // create vector of roles from all get methods
        Enumeration e = v.elements();

        while (e.hasMoreElements()) {
            Method getMethod = (Method) e.nextElement();
            LOG.debug("  identified 1-1 association method " + getMethod);

            String capitalizedName = javaBaseName(getMethod.getName());
            String beanName = Introspector.decapitalize(capitalizedName);
            Class returnType = getMethod.getReturnType();
            Class[] params = new Class[] { returnType };

            // look for set method
            Method setMethod = findMethod(OBJECT, SET_PREFIX + capitalizedName, void.class, params);
            if (setMethod == null) {
                setMethod = findMethod(OBJECT, "set_" + capitalizedName, void.class, params);
            }

            // look for associate
            Method associateMethod = findMethod(OBJECT, MODIFY_PREFIX + capitalizedName, void.class, params);
            if (associateMethod == null) {
                associateMethod = findMethod(OBJECT, "associate" + capitalizedName, void.class, params);
                if (associateMethod != null) {
                    LOG.warn("associate methods have been deprecated; use " + MODIFY_PREFIX + " instead: " + associateMethod);
                }
            }

            // look for dissociate
            Method dissociateMethod = findMethod(OBJECT, CLEAR_PREFIX + capitalizedName, void.class, NO_PARAMETERS_TYPES);
            if (dissociateMethod == null) {
                dissociateMethod = findMethod(OBJECT, "dissociate" + capitalizedName, void.class, params);
                if (dissociateMethod != null) {
                    LOG.warn("dissociate methods have been deprecated; use " + CLEAR_PREFIX + " instead: " + dissociateMethod);
                }
            }

            Method defaultMethod = findMethod(OBJECT, DEFAULT_PREFIX + capitalizedName, returnType, NO_PARAMETERS_TYPES);
            Method optionsMethod = findMethod(OBJECT, OPTIONS_PREFIX + capitalizedName, null, NO_PARAMETERS_TYPES);

            FieldMethods fieldMethods = new FieldMethods(getMethod, setMethod, associateMethod, dissociateMethod, defaultMethod,
                    optionsMethod);

            MethodPattern methodPattern = new MethodPattern(getMethod, capitalizedName, returnType);

            Method associateValidMethod = findMethod(JavaIntrospector.OBJECT, JavaIntrospector.VALID_PREFIX + capitalizedName,
                    String.class, methodPattern.getParameterTypes());

            DescriptiveMethods descriptiveMethods = findDescriptiveMethods(methodPattern);
            GeneralControlMethods controlMethods = findControlMethods(OBJECT, methodPattern, associateValidMethod, null);
            MemberSessionMethods sessionMethods = findSessionMethods(capitalizedName, params);

            LOG.debug("one-to-one association " + capitalizedName + " ->" + associateMethod);
            MemberIdentifier identifier = new MemberIdentifierImpl(className, beanName);
            boolean isPersisted = extension.isFieldPersisted(getMethod);
            boolean isDerived = setMethod == null;
            FieldFlags fieldFlags = setFieldFlags(capitalizedName, getMethod, params, isPersisted, isDerived);

            JavaOneToOneAssociation association = new JavaOneToOneAssociation(identifier, returnType, descriptiveMethods,
                    fieldMethods, controlMethods, fieldFlags, sessionMethods);

            final Annotation[] annotations = annotationFactorySet.process(getMethod);
            AnnotationUtil.addAnnotations(association, annotations);

            associations.addElement(association);
        }
    }

    FieldPeer[] orderArray(FieldPeer[] original, String[] order) {
        if (order == null) {
            return original;

        } else {
            for (int i = 0; i < order.length; i++) {
                order[i] = NameConvertor.simpleName(order[i]);
            }

            FieldPeer[] ordered = new FieldPeer[original.length];

            // work through each order element and find, if there is one, a
            // matching member.
            int orderedIndex = 0;
            ordering: for (int orderIndex = 0; orderIndex < order.length; orderIndex++) {
                for (int memberIndex = 0; memberIndex < original.length; memberIndex++) {
                    FieldPeer member = original[memberIndex];
                    if (member == null) {
                        continue;
                    }
                    if (member.getIdentifier().getName().equalsIgnoreCase(order[orderIndex])) {
                        ordered[orderedIndex++] = original[memberIndex];
                        original[memberIndex] = null;

                        continue ordering;
                    }
                }

                if (!order[orderIndex].trim().equals("")) {
                    LOG.warn("invalid ordering element '" + order[orderIndex] + "' in " + className);
                }
            }

            FieldPeer[] results = new FieldPeer[original.length];
            int index = 0;
            for (int i = 0; i < ordered.length; i++) {
                FieldPeer member = ordered[i];
                if (member != null) {
                    results[index++] = member;
                }
            }
            for (int i = 0; i < original.length; i++) {
                FieldPeer member = original[i];
                if (member != null) {
                    results[index++] = member;
                }
            }

            return results;
        }
    }

    Persistable persistable() {
        if (extension.isObjectTransient() || NonPersistable.class.isAssignableFrom(cls)) {
            return Persistable.TRANSIENT;
        } else if (extension.isObjectNotUserPersistable() || ProgramPersistable.class.isAssignableFrom(cls)) {
            return Persistable.PROGRAM_PERSISTABLE;
        } else {
            return Persistable.USER_PERSISTABLE;
        }
    }

    String pluralName() {
        String name = extension.getPluralName();
        if (name != null) {
            return name;
        }

        if (pluralNameMethod != null) {
            try {
                return (String) pluralNameMethod.invoke(null, NO_PARAMETERS);
            } catch (IllegalAccessException ignore) {} catch (InvocationTargetException ignore) {}
        }
        return NameConvertor.pluralName(singularName());
    }

    private String invokeSortOrderMethod(final String type) {
        Method method = findMethod(true, type + "Order", String.class, NO_PARAMETERS_TYPES);
        if (method == null) {
            return null;
        } else if (!Modifier.isStatic(method.getModifiers())) {
            LOG.warn("method " + className + "." + type + "Order() must be declared as static");
            return null;
        } else {
            String s;
            s = (String) invokeMethod(method, NO_PARAMETERS);
            if (s.trim().length() == 0) {
                return null;
            }
            return s;
        }
    }

    private Object invokeMethod(Method method, final Object[] parameters) {
        try {
            return method.invoke(null, parameters);
        } catch (IllegalAccessException ignore) {
            LOG.warn("method " + className + "." + method.getName() + "() must be declared as public");
            return null;
        } catch (InvocationTargetException e) {
            throw new ReflectionException(e);
        }
    }

    private void removeFields(final Class returnType) {
        findPrefixedMethods(OBJECT, GET_PREFIX, returnType, 0);
    }

    private void removeMethod(final boolean forClass, final String name, final Class returnType, final Class[] paramTypes) {
        findMethod(forClass, name, returnType, paramTypes);
    }

    String shortName() {
        String name = cls.getName();
        return name.substring(name.lastIndexOf('.') + 1);
    }

    String singularName() {
        if (singularNameMethod != null) {
            try {
                return (String) singularNameMethod.invoke(null, NO_PARAMETERS);
            } catch (IllegalAccessException ignore) {
                // ignore
            } catch (InvocationTargetException ignore) {
                // ignore
            }
        }

        String name = extension.getSingularName();
        if (name != null) {
            return name;
        }

        return NameConvertor.naturalName(shortName());
    }

    public String getDescription() {
        return extension.getDescription();
    }

    private NakedObjectSpecification specification(final Class returnType) {
        return NakedObjectsContext.getReflector().loadSpecification(returnType);
    }

    ObjectTitle title() {
        return title;
    }

    private void titleMethod() {
        Method titleMethod = findMethod(OBJECT, "title", String.class, null);
        if (titleMethod == null) {
            boolean hasMethod;
            try {
                Method toStringMethod = findMethod(OBJECT, "toString", String.class, null);
                if (toStringMethod == null) {
                    hasMethod = false;
                } else {
                    Class declaringClass = toStringMethod.getDeclaringClass();
                    hasMethod = declaringClass != Object.class;
                }
            } catch (Exception e) {
                hasMethod = false;
            }

            if (hasMethod) {
                title = new ObjectTitle() {
                    public String title(final NakedObject object) {
                        return object.getObject().toString();
                    }

                    public String toString() {
                        return "toString title";
                    }
                };
            } else {
                title = new ObjectTitle() {
                    public String title(final NakedObject object) {
                        return null;
                    }

                    public String toString() {
                        return "no title method";
                    }
                };
            }

        } else {
            title = new JavaObjectTitle(titleMethod);
        }
    }

    private Vector findValueFields(final Vector fields, final Class type, String prefix) {
        Vector v = findPrefixedMethods(OBJECT, prefix, type, 0);

        // create vector of attributes from all get methods
        for (Enumeration e = v.elements(); e.hasMoreElements();) {
            Method getMethod = (Method) e.nextElement();
            Class returnType = getMethod.getReturnType();
            String capitalizedName = javaBaseName(getMethod.getName());
            String beanName = Introspector.decapitalize(capitalizedName);

            Class[] params = new Class[] { returnType };

            Method setMethod = findMethod(OBJECT, SET_PREFIX + capitalizedName, void.class, params);
            if (setMethod == null) {
                setMethod = findMethod(OBJECT, "set_" + capitalizedName, void.class, params);
            }
            Method modifyMethod = findMethod(OBJECT, MODIFY_PREFIX + capitalizedName, void.class, params);
            Method clearMethod = findMethod(OBJECT, CLEAR_PREFIX + capitalizedName, void.class, NO_PARAMETERS_TYPES);

            Method optionsMethod = findMethod(OBJECT, OPTIONS_PREFIX + capitalizedName, null, NO_PARAMETERS_TYPES);

            Method defaultMethod = findMethod(OBJECT, DEFAULT_PREFIX + capitalizedName, type, NO_PARAMETERS_TYPES);

            // check for invalid methods
            // REVIEW should we check for invalid methods? What would be an invalid method?
            if (findMethod(OBJECT, "associate" + capitalizedName, void.class, params) != null) {
                LOG.error("the method associate" + capitalizedName + " is not needed in class " + className());
            }
            if (findMethod(OBJECT, "dissociate" + capitalizedName, void.class, params) != null) {
                LOG.error("the method dissociate" + capitalizedName + " is not needed in class " + className());
            }

            MethodPattern methodPattern = new MethodPattern(getMethod, capitalizedName, returnType);
            Method validMethod = findMethod(OBJECT, VALID_PREFIX + capitalizedName, String.class, methodPattern
                    .getParameterTypes());
            DescriptiveMethods descriptiveMethods = findDescriptiveMethods(methodPattern);
            GeneralControlMethods controlMethods = findControlMethods(OBJECT, methodPattern, validMethod, null);
            MemberSessionMethods sessionMethods = findSessionMethods(capitalizedName, params);

            boolean isPersisted = extension.isFieldPersisted(getMethod);
            boolean isDerived = setMethod == null;
            FieldFlags fieldFlags = setFieldFlags(capitalizedName, getMethod, params, isPersisted, isDerived);

            int maxLength = extension.getMaxLength(getMethod);
            int typicalLength = extension.getTypicalLength(getMethod);
            int noLines = extension.getNoLines(getMethod);
            if (noLines == 1 && returnType.isAssignableFrom(MultilineString.class)) {
                noLines = 6;
            }
            boolean canWrap = extension.canWrap(getMethod);
            String mask = extension.getMask(getMethod);
            FieldRegEx regEx = extension.getRegEx(getMethod);

            boolean canClear = !returnType.isPrimitive();
            
            // TODO allow for the extension to specify options

            // create Field
            LOG.debug("  identified value " + capitalizedName + " -> " + getMethod);
            MemberIdentifier identifier = new MemberIdentifierImpl(className, beanName);
            FieldMethods fieldMethods = new FieldMethods(getMethod, setMethod, modifyMethod, clearMethod, defaultMethod,
                    optionsMethod);
            JavaValueAssociation association = new JavaValueAssociation(identifier, getMethod.getReturnType(),
                    descriptiveMethods, fieldMethods, controlMethods, fieldFlags, sessionMethods, maxLength, typicalLength,
                    noLines, canClear, canWrap, mask, regEx);

            final Annotation[] annotations = annotationFactorySet.process(getMethod);
            AnnotationUtil.addAnnotations(association, annotations);

            fields.addElement(association);
        }

        return fields;
    }

    FieldFlags setFieldFlags(
            final String name,
            final Method getMethod,
            final Class[] params,
            final boolean isPersisted,
            final boolean isDerived) {
        When isHidden = isMemberHidden(name, getMethod, params);
        When isProtected = isMemberProtected(name, getMethod, params);
        boolean isOptional = isFieldOptional(name, getMethod, params);
        String businessKeyName = extension.getBusinessKeyName(getMethod);

        if (isDerived) {
            isProtected = When.ALWAYS;
        }
        return new FieldFlags(isHidden, isOptional, isProtected, isPersisted && !isDerived, businessKeyName);
    }

    ActionFlags setActionFlags(final String name, final Method getMethod, final Class[] params) {
        When isHidden = isMemberHidden(name, getMethod, params);
        When isProtected = isMemberProtected(name, getMethod, params);

        return new ActionFlags(isHidden, isProtected);
    }

    When isMemberHidden(final String name, final Method method, final Class[] params) {
        return checkWhen(extension.isHidden(method), name, HIDDEN_PREFIX, params);
    }

    boolean isFieldOptional(final String name, final Method method, final Class[] params) {
        boolean isOptional = checkFlag(extension.isFieldOptional(method), name, OPTIONAL_PREFIX, params);
        if (isOptional && method.getReturnType().isPrimitive()) {
            LOG.warn(getFullName() + "#" + name + " cannot be optional as it is a primitive; request ignored");
            return false;
        }
        return isOptional;
    }

    When isMemberProtected(final String name, final Method method, final Class[] params) {
        return checkWhen(extension.isFieldProtected(method), name, PROTECTED_PREFIX, params);
    }

    private boolean checkFlag(final boolean extensionFlag, final String name, String prefix, final Class[] params) {
        boolean isHidden;
        isHidden = extensionFlag;
        Method hiddenMethod = findMethod(CLASS, prefix + name, boolean.class, params);
        if (hiddenMethod != null) {
            // TODO this call and the one in the next if are too similar, factor them out
            if (((Boolean) invokeMethod(hiddenMethod, new Object[params.length])).booleanValue()) {
                isHidden = true;
            }
        }
        hiddenMethod = findMethod(CLASS, prefix + name, boolean.class, NO_PARAMETERS_TYPES);
        if (hiddenMethod != null) {
            if (((Boolean) invokeMethod(hiddenMethod, new Object[0])).booleanValue()) {
                isHidden = true;
            }
        }
        return isHidden;
    }

    private When checkWhen(final When when, final String name, String prefix, final Class[] params) {
        When isHidden;
        isHidden = when;
        Method hiddenMethod = findMethod(CLASS, prefix + name, boolean.class, params);
        if (hiddenMethod != null) {
            // TODO this call and the one in the next if are too similar, factor them out
            if (((Boolean) invokeMethod(hiddenMethod, new Object[params.length])).booleanValue()) {
                isHidden = When.ALWAYS;
            }
        }
        hiddenMethod = findMethod(CLASS, prefix + name, boolean.class, NO_PARAMETERS_TYPES);
        if (hiddenMethod != null) {
            if (((Boolean) invokeMethod(hiddenMethod, new Object[0])).booleanValue()) {
                isHidden = When.ALWAYS;
            }
        }
        return isHidden;
    }

    MemberSessionMethods findSessionMethods(final String name, final Class[] params) {
        Class[] sessionParams = new Class[params.length + 1];
        sessionParams[0] = UserMemento.class;
        System.arraycopy(params, 0, sessionParams, 1, params.length);
        Method sessionHideMethod = findMethod(CLASS, SESSION_HIDE_PREFIX + name, boolean.class, sessionParams);
        if (sessionHideMethod == null && params.length > 0) {
            sessionHideMethod = findMethod(CLASS, SESSION_HIDE_PREFIX + name, boolean.class, new Class[] { UserMemento.class });
        }

        Method sessionReadOnlyMethod = findMethod(CLASS, SESSION_READONLY_PREFIX + name, String.class, sessionParams);
        if (sessionReadOnlyMethod == null && params.length > 0) {
            sessionReadOnlyMethod = findMethod(CLASS, SESSION_READONLY_PREFIX + name, String.class,
                    new Class[] { UserMemento.class });
        }

        return new MemberSessionMethods(sessionHideMethod, sessionReadOnlyMethod);
    }

    protected Method getIconMethod() {
        return iconMethod;
    }

    public Method[] lifecycleMethods() {
        return lifecycleMethods;
    }

    public Method isPersistableMethod() {
        return canSaveMethod;
    }

    public boolean getQueryByExample() {
        return extension.queryByExample();
    }

    public Class getSearchViaRepository() {
        return extension.searchViaRepository();
    }

}
// Copyright (c) Naked Objects Group Ltd.
