package org.nakedobjects.nof.reflect.java.reflect;

import java.lang.reflect.Method;

import org.nakedobjects.noa.adapter.FieldRegEx;
import org.nakedobjects.noa.annotations.When;


public interface JavaIntrospectorExtension {

    /**
     * Handle <tt>@Named</tt> for classes.
     * @return
     */
    String getSingularName();

    /**
     * Handle <tt>@DescribedAs</tt> for classes.
     * @return
     */
    String getDescription();

    /**
     * Handle <tt>@Named</tt> for properties, collections and actions.
     * @return
     */
    MemberHelper getMemberName(final Method method);

    /**
     * Handle <tt>@Named</tt> for action parameters.
     */
    MemberHelper getParameterNames(final Method method);

    /**
     * Handle <tt>@Plural</tt> for classes.
     */
    String getPluralName();

    /**
     * Handle <tt>@DescribedAs</tt> for properties, collections and actions.
     */
    MemberHelper getMemberDescription(final Method method);

    /**
     * Handle <tt>@DescribedAs</tt> for action parameters
     */
    MemberHelper getParameterDescriptions(final Method method);

    boolean isAggregated();

    boolean isBounded();

    boolean isCached();

    boolean isImmutableOncePersisted();

    boolean isAlwaysImmutable();

    /**
     * Handle <tt>@FieldOrder</tt>
     * @return
     */
    String getFieldOrder();

    /**
     * Handle <tt>@ActionOrder</tt>
     */
    String getObjectActionOrder();

    /**
     * @deprecated
     */
    String getClassActionOrder();

    boolean isFieldOptional(Method method);

    MemberHelper getOptionalParameters(final Method method);

    boolean isFieldPersisted(Method method);

    When isActionDisabled(Method method);

    When isFieldProtected(Method method);

    /**
     * Handle <tt>@TypeOf</tt>.
     */
    Class getCollectionType(final Method method);

    boolean isDebug(Method method);

    boolean isExploration(Method method);

    boolean isLocal(Method method);

    boolean isRemote(Method method);

    When isHidden(Method method);

    int getMaxLength(Method method);

    int[] getParametersMaxLengths(Method method);

    int getTypicalLength(Method method);

    int[] getParametersTypicalLengths(Method method);

    int getNoLines(Method method);

    int[] getParametersNoLines(Method method);

    boolean canWrap(Method method);

    boolean[] getParametersCanWrap(Method method);

    FieldRegEx getRegEx(Method method);

    String getMask(Method method);

    boolean isSearchable();

    boolean queryByExample();

    Class searchViaRepository();

    String getBusinessKeyName(Method method);

    boolean isEntity();

    /**
     * Not sure what the intent is here...
     */
    MemberHelper isUnusable(Method method);

    /**
     * Not sure what the intent is here...
     */
    MemberHelper getParameterDefaults(final Method method);

    /**
     * Not sure what the intent is here...
     */
    MemberHelper getParameterOptions(Method method);

}

// Copyright (c) Naked Objects Group Ltd.
