package org.nakedobjects.nof.reflect.java.reflect;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.nakedobjects.nof.reflect.peer.ReflectionException;


public class MethodHelper implements MemberHelper {

    private final Method method;

    public MethodHelper(Method method) {
        this.method = method;
    }

    public Object execute(Object target) {
        try {
            if (target == null || method.getDeclaringClass().isAssignableFrom(target.getClass())) {
                Class[] parameterTypes = method.getParameterTypes();
                Object[] parameters = new Object[parameterTypes.length];
                injectPrimitiveParameters(parameters, parameterTypes);
                return method.invoke(target, parameters);
            } else {
                throw new ReflectionException("Can't invoke " + method.getName() + " on object " + target);
            }
        } catch (IllegalAccessException ignore) {
            // REVIEW should we ignore this
            throw new ReflectionException("method " + method.getDeclaringClass().getName() + "." + method.getName()
                    + "() must be declared as public", ignore);
        } catch (InvocationTargetException e) {
            if (e.getTargetException() instanceof RuntimeException) {
                throw (RuntimeException) e.getTargetException();
            }
            throw new ReflectionException(e.getTargetException());
        }
    }

    private void injectPrimitiveParameters(Object[] parameters, Class[] types) {
        for (int i = 0; i < parameters.length; i++) {
            if (types[i].isPrimitive()) {
                if (types[i].equals(boolean.class)) {
                    parameters[i] = new Boolean(false);
                } else if (types[i].equals(byte.class)) {
                    parameters[i] = new Byte((byte) 0);
                } else if (types[i].equals(short.class)) {
                    parameters[i] = new Short((short) 0);
                } else if (types[i].equals(int.class)) {
                    parameters[i] = new Integer(0);
                } else if (types[i].equals(long.class)) {
                    parameters[i] = new Long(0);
                } else if (types[i].equals(float.class)) {
                    parameters[i] = new Float(0);
                } else if (types[i].equals(double.class)) {
                    parameters[i] = new Double(0);
                } else if (types[i].equals(char.class)) {
                    parameters[i] = new Character(' ');
                }
            }
        }
    }

    public String toString() {
        return "method: " + method;
    }

    Method getMethod() {
        return method;
    }

}

// Copyright (c) Naked Objects Group Ltd.
