package org.nakedobjects.nof.reflect.java.reflect;

import java.util.Comparator;

/**
 * Compares by (simple) group name of each {@link OrderSet}.
 * 
 * <p>
 * Note that it only makes sense to use this comparator for
 * {@link OrderSet}s that are known to have the same parent
 * {@link OrderSet}s.
 */
public class OrderSetGroupNameComparator implements Comparator {

    public OrderSetGroupNameComparator(boolean ensureInSameGroupPath) {
        this.ensureInSameGroupPath = ensureInSameGroupPath;
    }
    
    private final boolean ensureInSameGroupPath;
    
    public int compare(
            Object o1,
            Object o2) {
        return compare((OrderSet)o1, ((OrderSet)o2));
    }

    public int compare(
            OrderSet o1,
            OrderSet o2) {
        if (ensureInSameGroupPath && !o1.getGroupPath().equals(o2.getGroupPath())) {
            throw new IllegalArgumentException("OrderSets being compared do not have the same group path");
        }
        
        String groupName1 = o1.getGroupName();
        String groupName2 = o2.getGroupName();
        
        return groupName1.compareTo(groupName2);
    }
}
