package org.nakedobjects.nof.reflect.java.reflect;

import java.util.StringTokenizer;

import org.nakedobjects.nof.core.util.NameConvertor;
import org.nakedobjects.nof.reflect.peer.MemberPeer;


class SimpleOrderSet extends OrderSet {
    static SimpleOrderSet createOrderSet(final String order, final MemberPeer[] members) {
        SimpleOrderSet set = new SimpleOrderSet(members);

        StringTokenizer st = new StringTokenizer(order, ",");
        while (st.hasMoreTokens()) {
            String element = st.nextToken().trim();

            boolean ends;
            if (ends = element.endsWith(")")) {
                element = element.substring(0, element.length() - 1).trim();
            }

            if (element.startsWith("(")) {
                int colon = element.indexOf(':');
                String groupName = element.substring(1, colon).trim();
                element = element.substring(colon + 1).trim();
                set = set.createSubOrderSet(groupName, element);
            } else {
                set.add(element);
            }

            if (ends) {
                set = set.parent;
            }
        }
        set.addAnyRemainingMember();
        return set;
    }

    private final SimpleOrderSet parent;
    private final MemberPeer[] members;

    private SimpleOrderSet(MemberPeer[] members) {
        super("");
        this.members = members;
        parent = null;
    }

    private SimpleOrderSet(final SimpleOrderSet set, final String groupName, final String name, MemberPeer[] members) {
        super(groupName);
        parent = set;
        parent.addElement(this);
        this.members = members;
        add(name);
    }

    private void add(String name) {
        MemberPeer memberWithName = getMemberWithName(name);
        if (memberWithName != null) {
            addElement(memberWithName);
        }
    }

    private void addAnyRemainingMember() {
        for (int i = 0; i < members.length; i++) {
            if (members[i] != null) {
                addElement(members[i]);
            }
        }

    }

    private SimpleOrderSet createSubOrderSet(String groupName, String memberName) {
        return new SimpleOrderSet(this, groupName, memberName, members);
    }

    private MemberPeer getMemberWithName(final String name) {
        String searchName = NameConvertor.simpleName(name);
        for (int i = 0; i < members.length; i++) {
            MemberPeer member = members[i];
            if (member != null) {
                String testName = NameConvertor.simpleName(member.getIdentifier().getName());
                if (testName.equals(searchName)) {
                    members[i] = null;
                    return member;
                }
            }
        }
        return null;
    }

}

// Copyright (c) Naked Objects Group Ltd.
