package org.nakedobjects.nof.reflect.java.value;

import org.nakedobjects.applib.value.Color;
import org.nakedobjects.noa.adapter.TextEntryParseException;
import org.nakedobjects.noa.adapter.value.ColorValue;
import org.nakedobjects.nof.core.adapter.value.AbstractValueAdapter;


public class ColorAdapter extends AbstractValueAdapter implements ColorValue {
    private Color color;

    public ColorAdapter() {
        this.color = Color.WHITE;
    }

    public ColorAdapter(final Color color) {
        this.color = color;
    }

    public void clear() {
        this.color = null;
    }

    public int color() {
        return color.intValue();
    }

    public String getIconName() {
        return "color";
    }

    public Object getObject() {
        return color;
    }

    public Class getValueClass() {
        return color.getClass();
    }

    protected void doParse(String text) {
        try {
            if (text.startsWith("0x")) {
                setColor(Integer.parseInt(text.substring(2), 16));
            } else if (text.startsWith("#")) {
                setColor(Integer.parseInt(text.substring(1), 16));
            } else {
                setColor(Integer.parseInt(text));
            }
        } catch (NumberFormatException e) {
            throw new TextEntryParseException("Not a number " + text, e);
        }
    }

    protected String doEncode() {
        return String.valueOf(color());
    }

    protected void doRestore(String data) {
        setColor(Integer.valueOf(data).intValue());
    }

    public void setColor(final int color) {
        this.color = new Color(color);
    }

    public void setMask(String mask) {}

    public String titleString() {
        return color.title();
    }

    public String toString() {
        return "ColorAdapter #" + Integer.toHexString(color()).toUpperCase();
    }

    public boolean isEmpty() {
        return false;
    }
}
// Copyright (c) Naked Objects Group Ltd.
