package org.nakedobjects.nof.reflect.java.value;

import org.nakedobjects.applib.value.MultilineString;
import org.nakedobjects.noa.adapter.value.MultilineStringValue;
import org.nakedobjects.nof.core.adapter.value.AbstractValueAdapter;
import org.nakedobjects.nof.core.util.ToString;


public class MultilineStringAdapter extends AbstractValueAdapter implements MultilineStringValue {
    private MultilineString string;

    public MultilineStringAdapter() {
        this.string = null;
    }

    public MultilineStringAdapter(final MultilineString string) {
        this.string = string;
    }

    protected String doEncode() {
        String text = stringValue();
        if (text.equals("NULL") || isEscaped(text)) {
            return escapeText(text);
        } else {
            return text;
        }
    }

    private boolean isEscaped(final String text) {
        return text.startsWith("/");
    }

    private String escapeText(final String text) {
        return "/" + text;
    }

    public boolean canClear() {
        return true;
    }

    public void clear() {
        string = null;
    }

    public String getIconName() {
        return "text";
    }

    public Object getObject() {
        return string;
    }

    public Class getValueClass() {
        return MultilineString.class;
    }

    public boolean isEmpty() {
        return string == null;
    }
    
    protected void doParse(String text) {
        string = new MultilineString(text);
    }
    
    protected void doRestore(String data) {
        if (isEscaped(data)) {
            setValue(data.substring(1));
        } else {
            setValue(data);
        }
    }

    public void setMask(String mask) {}

    public void setValue(final String value) {
        string = new MultilineString(value);
    }

    public String stringValue() {
        return string == null ? null: string.getString();
    }

    public String titleString() {
        return string == null ? "" : string.getString();
    }

    public String toString() {
        ToString str = new ToString(this);
        str.append("string", string == null ? "" : string.getString());
        return str.toString();
    }
}
// Copyright (c) Naked Objects Group Ltd.
