package org.nakedobjects.nof.reflect.java.value;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.ParseException;

import org.nakedobjects.applib.value.Percentage;
import org.nakedobjects.noa.adapter.TextEntryParseException;
import org.nakedobjects.noa.adapter.value.FloatingPointValue;
import org.nakedobjects.nof.core.adapter.value.AbstractValueAdapter;
import org.nakedobjects.nof.core.conf.Configuration;
import org.nakedobjects.nof.core.context.NakedObjectsContext;


public class PercentageAdapter extends AbstractValueAdapter implements FloatingPointValue {
    private static final NumberFormat PERCENTAGE_FORMAT = NumberFormat.getPercentInstance();
    private static final NumberFormat DECIMAL_FORMAT = NumberFormat.getNumberInstance();
    private NumberFormat format = PERCENTAGE_FORMAT;
    private Percentage percentage;

    public PercentageAdapter() {
        String formatRequired = NakedObjectsContext.getConfiguration().getString(Configuration.ROOT + "value.format.percentage");
        if (formatRequired == null) {
            format = PERCENTAGE_FORMAT;
        } else {
            setMask(formatRequired);
        }
    }

    public PercentageAdapter(Percentage value) {
        this();
        percentage = value;
    }

    protected String doEncode() {
        return String.valueOf(floatValue());
    }

    public boolean canClear() {
        return true;
    }

    public void clear() {
        percentage = null;
    }

    public Float floatValue() {
        return new Float(percentage.floatValue());
    }

    public String getIconName() {
        return "percentage";
    }

    public Object getObject() {
        return percentage;
    }

    public Class getValueClass() {
        return Percentage.class;
    }

    public boolean isEmpty() {
        return percentage == null;
    }

    protected void doParse(String text) {
        try {
            setValue(new Float(format.parse(text).floatValue()));
        } catch (ParseException e) {
            try {
                setValue(new Float(DECIMAL_FORMAT.parse(text).floatValue()));
            } catch (ParseException ee) {
                throw new TextEntryParseException("Not a number " + text, ee);
            }
        }
    }

    protected void doRestore(String data) {
        percentage = new Percentage(Float.valueOf(data).floatValue());
    }

    public void setMask(String mask) {
        format = new DecimalFormat(mask);
    }

    public void setValue(Float value) {
        percentage = new Percentage(value.floatValue());
    }

    public String titleString() {
        return isEmpty() ? "" : format.format(percentage.floatValue());
    }

    public String toString() {
        return "PercentageAdapter: " + percentage;
    }

}

// Copyright (c) Naked Objects Group Ltd.
