package org.nakedobjects.nof.reflect.java.value;

import org.nakedobjects.applib.value.TimeStamp;
import org.nakedobjects.noa.adapter.InvalidEntryException;
import org.nakedobjects.noa.adapter.value.TimeStampValue;
import org.nakedobjects.nof.core.adapter.value.AbstractTemporalValueAdapter;
import org.nakedobjects.nof.core.conf.Configuration;
import org.nakedobjects.nof.core.context.NakedObjectsContext;

import java.text.DateFormat;
import java.util.Date;
import java.util.Hashtable;


public class TimeStampAdapter extends AbstractTemporalValueAdapter implements TimeStampValue {
    private static Hashtable formats = new Hashtable();

    static {
        formats.put("iso", createDateFormat("yyyy-MM-dd HH:mm:ss.SSS"));
        formats.put(ISO_ENCODING_FORMAT, createDateFormat("yyyyMMdd'T'HHmmssSSS"));
        formats.put("medium", DateFormat.getDateTimeInstance(DateFormat.MEDIUM, DateFormat.LONG));
        formats.put("short", DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.LONG));
    }

    private TimeStamp timeStamp;

    public TimeStampAdapter() {
        super("timestamp");

        String formatRequired = NakedObjectsContext.getConfiguration().getString(Configuration.ROOT + "value.format.timestamp");
        if (formatRequired == null) {
            format = (DateFormat) formats().get(defaultFormat());
        } else {
            setMask(formatRequired);
        }
    }

    public TimeStampAdapter(TimeStamp timeStamp) {
        this();
        this.timeStamp = timeStamp;
    }

    protected void add(int years, int months, int days, int hours, int minutes) {}

    public void clear() {
        timeStamp = null;
    }

    protected Date dateValue() {
        return getDate();
    }

    protected String defaultFormat() {
        return "short";
    }

    public int defaultTypicalLength() {
        return 18;
    }

    protected Hashtable formats() {
        return formats;
    }

    private Date getDate() {
        return timeStamp == null ? null : new Date(timeStamp.longValue());
    }

    public String getIconName() {
        return "timestamp";
    }

    public Object getObject() {
        return timeStamp;
    }

    public Class getValueClass() {
        return TimeStamp.class;
    }

    /**
     * Return true if the time stamp is blank
     */
    public boolean isEmpty() {
        return isNull();
    }

    private boolean isNull() {
        return timeStamp == null;
    }

    protected void now() {
        throw new InvalidEntryException("Can't change a timestamp.");
    }

    public void parseTextEntry(String text) {
        throw new InvalidEntryException("Can't change a timestamp.");
    }

    protected void setDate(Date date) {
        timeStamp = new TimeStamp(date.getTime());
    }

    public java.util.Date timeValue() {
        return isNull() ? null : getDate();
    }

    public String toString() {
        return "TimeStampAdapter: " + timeStamp;
    }

}

// Copyright (c) Naked Objects Group Ltd.
